### ms-msrp_msgsvcsend.idl
"""
RPC definitions for the following interfaces:
- msgsvcsend (v1.0): 5a7b91f8-ff00-11d0-a9b2-00c04fb6e6fc
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfVarStrNullField,
    NDRIntField,
    register_dcerpc_interface,
)


class NetrSendMessage_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullField("From", ""),
        NDRConfVarStrNullField("To", ""),
        NDRConfVarStrNullField("Text", ""),
    ]


class NetrSendMessage_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


MSGSVCSEND_OPNUMS = {0: DceRpcOp(NetrSendMessage_Request, NetrSendMessage_Response)}
register_dcerpc_interface(
    name="msgsvcsend",
    uuid=uuid.UUID("5a7b91f8-ff00-11d0-a9b2-00c04fb6e6fc"),
    version="1.0",
    opnums=MSGSVCSEND_OPNUMS,
)
