### ms-adts_Claims.idl
"""
RPC definitions for the following interfaces:
- Claims (v0.0): None
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
)
