### ms-ssp.idl
"""
RPC definitions for the following interfaces:
- ISingleSignonRemoteMasterSecret (v1.0): 9D07CA0D-8F02-4ed5-B727-ACF37FEA5BBC
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRIntField,
    register_dcerpc_interface,
)


class RemoteGetMasterSecret_Request(NDRPacket):
    fields_desc = [NDRIntField("pcbSecret", 0)]


class RemoteGetMasterSecret_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcbSecret", None, size_of="pbSecret"),
        NDRConfStrLenField("pbSecret", "", size_is=lambda pkt: pkt.pcbSecret),
        NDRIntField("status", 0),
    ]


ISINGLESIGNONREMOTEMASTERSECRET_OPNUMS = {
    0: DceRpcOp(RemoteGetMasterSecret_Request, RemoteGetMasterSecret_Response)
}
register_dcerpc_interface(
    name="ISingleSignonRemoteMasterSecret",
    uuid=uuid.UUID("9D07CA0D-8F02-4ed5-B727-ACF37FEA5BBC"),
    version="1.0",
    opnums=ISINGLESIGNONREMOTEMASTERSECRET_OPNUMS,
)
