### ms-wdsc.idl
"""
RPC definitions for the following interfaces:
- WdsRpcInterface (v1.0): 1A927394-352E-4553-AE3F-7CF4AAFCA620
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRIntField,
    register_dcerpc_interface,
)


class WdsRpcMessage_Request(NDRPacket):
    fields_desc = [
        NDRIntField("uRequestPacketSize", None, size_of="bRequestPacket"),
        NDRConfStrLenField(
            "bRequestPacket", "", size_is=lambda pkt: pkt.uRequestPacketSize
        ),
    ]


class WdsRpcMessage_Response(NDRPacket):
    fields_desc = [
        NDRIntField("puReplyPacketSize", None, size_of="pbReplyPacket"),
        NDRConfStrLenField(
            "pbReplyPacket", "", size_is=lambda pkt: pkt.puReplyPacketSize
        ),
        NDRIntField("status", 0),
    ]


WDSRPCINTERFACE_OPNUMS = {0: DceRpcOp(WdsRpcMessage_Request, WdsRpcMessage_Response)}
register_dcerpc_interface(
    name="WdsRpcInterface",
    uuid=uuid.UUID("1A927394-352E-4553-AE3F-7CF4AAFCA620"),
    version="1.0",
    opnums=WDSRPCINTERFACE_OPNUMS,
)
